﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.Notifications;
using Hotline.SeedData;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Users;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Utility.SequentialId;

namespace Hotline.FlowEngine.Workflows
{
    public class WorkflowDomainService : IWorkflowDomainService, IScopeDependency
    {
        private readonly IWorkflowRepository _workflowRepository;
        private readonly IWorkflowStepRepository _workflowStepRepository;
        private readonly IWorkflowTraceRepository _workflowTraceRepository;
        private readonly IWorkflowSupplementRepository _workflowSupplementRepository;
        private readonly IWorkflowAssignRepository _workflowAssignRepository;
        private readonly IWorkflowCountersignRepository _workflowCountersignRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IMapper _mapper;
        private readonly IMediator _mediator;
        private readonly ILogger<WorkflowDomainService> _logger;

        public WorkflowDomainService(
            IWorkflowRepository workflowRepository,
            IWorkflowStepRepository workflowStepRepository,
            IWorkflowTraceRepository workflowTraceRepository,
            IWorkflowSupplementRepository workflowSupplementRepository,
            IWorkflowAssignRepository workflowAssignRepository,
            IWorkflowCountersignRepository workflowCountersignRepository,
            ISessionContext sessionContext,
            IMapper mapper,
            IMediator mediator,
            ILogger<WorkflowDomainService> logger)
        {
            _workflowRepository = workflowRepository;
            _workflowStepRepository = workflowStepRepository;
            _workflowTraceRepository = workflowTraceRepository;
            _workflowSupplementRepository = workflowSupplementRepository;
            _workflowAssignRepository = workflowAssignRepository;
            _workflowCountersignRepository = workflowCountersignRepository;

            _sessionContext = sessionContext;
            _mapper = mapper;
            _mediator = mediator;
            _logger = logger;
        }

        public async Task<Workflow> CreateWorkflowAsync(Definition definition, string title, CancellationToken cancellationToken)
        {
            var workflow = new Workflow
            {
                Title = title,
                ModuleId = definition.ModuleId,
                ModuleName = definition.ModuleName,
                ModuleCode = definition.ModuleCode,
                DefinitionId = definition.Id,
                Status = EWorkflowStatus.Runnable,
                TimeLimit = GetTimeLimit(definition.Code),
                ExpiredTime = GenerateExpiredTime(definition.Code),
                StepBoxes = new(),
                Traces = new(),
                Definition = definition
            };

            workflow.AssignTime = workflow.CreationTime;

            await _workflowRepository.AddAsync(workflow, cancellationToken);

            return workflow;
        }

        /// <summary>
        /// 流程开始
        /// </summary>
        /// <param name="workflow"></param>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task StartAsync(Workflow workflow, BasicWorkflowDto dto, StepDefine nextStepBoxDefine, FlowAssignMode flowAssignMode, CancellationToken cancellationToken)
        {
            //var nextStepBoxDefine = GetStepBoxDefine(workflow.Definition, dto.NextStepCode);

            var isStartCountersign = nextStepBoxDefine.IsStartCountersign(dto.NextHandlers.Count);
            //检查是否支持会签
            if (isStartCountersign && nextStepBoxDefine.CountersignMode == ECountersignMode.UnSupport)
                throw new UserFriendlyException($"当前节点不支持会签, defineCode: {workflow.Definition.Code}", "当前节点不支持会签");

            //1. 如果不是按角色指派，handlers必填 2. 如果按角色指派，handlers可以不选
            if (nextStepBoxDefine.HandlerType is not EHandlerType.Role && !dto.NextHandlers.Any())
                throw UserFriendlyException.SameMessage("未指派办理人");

            //开始节点
            var (startStepBox, startStep) = await CreateStartStepAsync(workflow, dto, isStartCountersign, cancellationToken);

            if (isStartCountersign)
            {
                //创建会签数据
                var countersign = await CreateCountersignAsync(workflow.Id, startStep.Id, startStep.Code, dto.NextHandlers.Count, startStep.CountersignId, cancellationToken);
                startStep.StartCountersignId = countersign.Id;
                await _workflowStepRepository.UpdateAsync(startStep, cancellationToken);
            }

            //开始节点trace
            await AcceptTraceAsync(workflow, startStepBox, startStep, cancellationToken);
            await NextTraceAsync(workflow, dto, startStep, cancellationToken);

            //第二节点（创建即为 已指派/待接办 状态）
            var nextStepBox = await CreateStepAsync(isStartCountersign, workflow, nextStepBoxDefine, dto, EWorkflowStepStatus.Assigned,
                startStepBox, startStep, cancellationToken);

            //更新当前节点名称、时间、会签节点code 等字段
            workflow.SetWorkflowCurrentStepInfo(isStartCountersign, nextStepBox);

            workflow.UpdateHandlers(_sessionContext.RequiredUserId, _sessionContext.RequiredOrgCode,
                flowAssignMode.FlowAssignType, flowAssignMode.Handlers);

            await _workflowRepository.UpdateAsync(workflow, cancellationToken);

            //publish
            _mediator.Publish(new StartWorkflowNotify(workflow, dto, isStartCountersign, flowAssignMode));
        }

        public async Task<Workflow> GetWorkflowAsync(string workflowId,
            bool withDefine = false, bool withSteps = false,
            bool withTraces = false, bool withSupplements = false,
            bool withAssigns = false, bool withCountersigns = false,
            CancellationToken cancellationToken = default)
        {
            var query = _workflowRepository.Queryable().Where(d => d.Id == workflowId);
            if (withDefine)
                query = query.Includes(d => d.Definition);
            if (withSupplements)
                query = query.Includes(d => d.Supplements, d => d.Creator);
            if (withAssigns)
                query = query.Includes(d => d.Assigns);
            if (withCountersigns)
                query = query.Includes(d => d.Countersigns);

            var workflow = await query.FirstAsync();
            if (workflow is null)
                throw new UserFriendlyException("无效workflowId");

            if (withSteps)
            {
                var steps = await _workflowStepRepository.Queryable()
                    .Where(d => d.WorkflowId == workflow.Id)
                    .OrderBy(d => d.CreationTime)
                    .ToTreeAsync(d => d.Steps, d => d.ParentId, null);
                workflow.StepBoxes = steps;
            }

            if (withTraces)
            {
                var traces = await _workflowTraceRepository.Queryable()
                    .Where(d => d.WorkflowId == workflow.Id)
                    .OrderBy(d => d.CreationTime)
                    .ToTreeAsync(d => d.Traces, d => d.ParentId, null);
                workflow.Traces = traces;
            }


            return workflow;
        }

        /// <summary>
        /// 受理(接办)
        /// </summary>
        public async Task AcceptAsync(Workflow workflow, string userId, string userName, string orgCode, string orgName, CancellationToken cancellationToken)
        {
            if (!workflow.CanHandle(_sessionContext.RequiredUserId, _sessionContext.RequiredOrgCode)) return;
            //工单完成以后查看的场景
            if (workflow.Status is not EWorkflowStatus.Runnable) return;

            var (currentStepBox, currentStep) = GetUnCompleteStepOrDefault(workflow.StepBoxes, orgCode, userId);
            if (currentStep is null) return;
            if (currentStep.Status is EWorkflowStepStatus.Accepted) return;
            if (currentStep.HandlerType is EHandlerType.AssignUser or EHandlerType.Role)
            {
                //userId
                if (!currentStep.HandlerIds.Contains(userId)) return;
            }
            else
            {
                //orgId
                if (!currentStep.HandlerIds.Contains(orgCode)) return;
            }
            if (currentStep.StepType is EStepType.End)
                throw new UserFriendlyException("当前流程已流转到最终步骤");

            var changedSteps = new List<WorkflowStep> { currentStep };
            if (currentStepBox.Status is EWorkflowStepStatus.Assigned)
            {
                currentStepBox.Status = EWorkflowStepStatus.Accepted;
                changedSteps.Add(currentStepBox);
            }
            currentStep.Accept(userId, userName, _sessionContext.RequiredOrgCode, _sessionContext.OrgName);

            //接办时非会签并且有多个接办部门时需更新接办部门
            if (!workflow.IsInCountersign())
            {
                var assigns = await _workflowAssignRepository.QueryAsync(d => d.WorkflowId == workflow.Id);
                if (assigns.Count > 1)
                {
                    await _workflowAssignRepository.RemoveRangeAsync(assigns, cancellationToken);

                    var assign = WorkflowAssign.Create(workflow.Id, orgCode, orgName);
                    await _workflowAssignRepository.AddAsync(assign, cancellationToken);
                }
            }

            await _workflowStepRepository.UpdateRangeAsync(changedSteps, cancellationToken);

            await AcceptTraceAsync(workflow, currentStepBox, currentStep, cancellationToken);

            await _mediator.Publish(new AcceptWorkflowNotify(workflow), cancellationToken);
        }

        /// <summary>
        /// 办理（流转至下一节点）
        /// </summary>
        public async Task NextAsync(Workflow workflow, BasicWorkflowDto dto, StepDefine nextStepBoxDefine,
            bool isOutOfCallCenter, bool isStartCountersign, FlowAssignMode flowAssignMode, CancellationToken cancellationToken)
        {
            ValidatePermission(workflow);
            CheckWhetherRunnable(workflow.Status);

            #region 办理当前节点

            var (currentStepBox, currentStep) = GetUnCompleteStep(workflow.StepBoxes, _sessionContext.RequiredOrgCode, _sessionContext.RequiredUserId);
            if (currentStep.Status is EWorkflowStepStatus.Completed or EWorkflowStepStatus.Created)
                throw UserFriendlyException.SameMessage("当前节点状态无法办理");
            if (currentStep.Status is EWorkflowStepStatus.Assigned)
                await AcceptAsync(workflow,
                    _sessionContext.RequiredUserId,
                    _sessionContext.UserName,
                    _sessionContext.RequiredOrgCode,
                    _sessionContext.OrgName,
                    cancellationToken);
            if (currentStep.StepType is EStepType.End)
                throw new UserFriendlyException("当前流程已流转到最终步骤");

            //检查是否支持会签办理
            if (isStartCountersign && nextStepBoxDefine.CountersignMode == ECountersignMode.UnSupport)
                throw UserFriendlyException.SameMessage($"下一节点不支持会签办理, code: {currentStep.Code}");
            WorkflowCountersign? countersign = null;
            if (isStartCountersign)
            {
                //创建会签数据
                countersign = await CreateCountersignAsync(workflow.Id, currentStep.Id, currentStep.Code,
                    dto.NextHandlers.Count, currentStep.CountersignId, cancellationToken);
                currentStep.StartCountersign(countersign.Id);
            }


            _mapper.Map(dto, currentStep);

            //step办理状态
            currentStep.StepComplete(
                _sessionContext.RequiredUserId, _sessionContext.UserName,
                _sessionContext.RequiredOrgCode, _sessionContext.OrgName,
                dto.NextStepCode);

            //stepBox办理状态
            currentStepBox.CheckStepBoxStatusAndUpdate();

            var updateSteps = new List<WorkflowStep> { currentStepBox, currentStep };
            //结束当前会签流程
            if (currentStep.StepType is EStepType.CountersignEnd && currentStep.IsInCountersign)
            {
                var countersignStartStepBox =
                    workflow.StepBoxes.FirstOrDefault(d => d.Code == currentStep.CountersignStartCode);
                if (countersignStartStepBox is null)
                    throw new UserFriendlyException(
                    $"未查询到会签开始stepBox, workflowId: {workflow.Id}, stepCode: {currentStep.CountersignStartCode}", "未查询到会签开始节点");
                var countersignStartStep =
                    countersignStartStepBox.Steps.FirstOrDefault(d => d.HasStartCountersign && d.StartCountersignId == currentStep.CountersignId);
                if (countersignStartStep != null)//如果==null，说明未发起会签是继承的外层会签
                {
                    //结束step会签信息
                    countersignStartStep.CountersignComplete();
                    updateSteps.Add(countersignStartStep);

                    //结束会签
                    var currentCountersign = workflow.Countersigns.FirstOrDefault(d => d.Id == currentStep.CountersignId);
                    if (currentCountersign is null)
                        throw new UserFriendlyException(
                            $"未查询到对应会签信息，workflowId:{workflow.Id}, countersignId:{currentStep.CountersignId}", "无效会签编号");
                    currentCountersign.Complete(currentStep.Id, currentStep.Code);
                    await _workflowCountersignRepository.UpdateAsync(currentCountersign, cancellationToken);
                }
            }

            await _workflowStepRepository.UpdateRangeAsync(updateSteps, cancellationToken);

            #endregion

            #region 处理流程

            //检查会签是否结束，并更新当前会签节点字段
            var isCountersignOver = false;
            if (currentStep.StepType is EStepType.CountersignEnd && workflow.IsInCountersign())
            {
                isCountersignOver = workflow.CheckIfCountersignOver();
                if (isCountersignOver)
                    workflow.EndCountersign();
            }

            //检查是否流转到流程终点
            if (nextStepBoxDefine.StepType is EStepType.End && !workflow.IsInCountersign())
            {
                workflow.Complete();
                await _workflowRepository.UpdateAsync(workflow, cancellationToken);

                await _mediator.Publish(new EndWorkflowNotify(workflow), cancellationToken);
                return;
            }

            //是否从中心流转出去，重新计算expiredTime 
            if (isOutOfCallCenter)
            {
                workflow.IsStraight = false;
                workflow.ExpiredTime = GenerateExpiredTime(workflow.Definition.Code);
                workflow.AssignTime = DateTime.Now;
            }

            //最终办理意见与时间处理
            if (currentStep.StepType is EStepType.Normal && nextStepBoxDefine.StepType is EStepType.CountersignEnd)
            {
                //最终办理
                workflow.Opinion = dto.Opinion;
                await _mediator.Publish(new OrderFinalManageNotify(workflow), cancellationToken);
            }
            else if (currentStep.StepType is EStepType.CountersignEnd && nextStepBoxDefine.StepType is EStepType.Normal)
            {
                //汇总以后又重新指派到非汇总节点办理
                workflow.ResetOption();
                await _mediator.Publish(new OrderRecallFinalManageNotify(workflow), cancellationToken);
            }

            //创建下一节点(会签汇总节点不重复创建)
            var nextStepBox = await CreateStepAsync(isStartCountersign, workflow, nextStepBoxDefine, dto, EWorkflowStepStatus.Created,
                currentStepBox, currentStep, cancellationToken);

            //下一节点为汇总节点时，检查下一节点是否可办理
            var nextStepCanHandle = true;
            if (nextStepBox.StepType is EStepType.CountersignEnd)
            {
                if (currentStep.IsInCountersign)
                {
                    //同一会签Id，非汇总节点
                    var steps = await _workflowStepRepository.QueryAsync(d =>
                        d.WorkflowId == workflow.Id
                        && d.CountersignId == currentStep.CountersignId
                        && d.StepType != EStepType.End);
                    //(当前办理节点所处同一会签内的所有step全都办理完成并且如果开启了会签的step，必须会签结束)
                    var unComplete = steps.Any(d =>
                        d.Status != EWorkflowStepStatus.Completed ||
                        (d.HasStartCountersign && !d.IsStartedCountersignComplete.Value));
                    nextStepCanHandle = !unComplete;
                }
            }
            if (nextStepCanHandle)
            {
                //将下一节点处理为已指派/可接办
                await SetNextCountersignEndAssignedAsync(nextStepBox, currentStep, cancellationToken);
                _mediator.Publish(new CountersignEndAssigned(workflow), cancellationToken);
            }


            //更新workflow当前节点名称、时间、会签节点code 等字段
            workflow.SetWorkflowCurrentStepInfo(isStartCountersign, nextStepBox);

            workflow.UpdateHandlers(_sessionContext.RequiredUserId, _sessionContext.RequiredOrgCode,
                flowAssignMode.FlowAssignType, flowAssignMode.Handlers);

            await _workflowRepository.UpdateAsync(workflow, cancellationToken);

            #endregion

            #region 流转记录

            await NextTraceAsync(workflow, dto, currentStep, cancellationToken);

            #endregion

            _mediator.Publish(new NextStepNotify(workflow, dto, isStartCountersign, isCountersignOver, flowAssignMode));
        }

        /// <summary>
        /// 退回（返回前一节点）
        /// </summary>
        /// <returns></returns>
        public async Task PreviousAsync(Workflow workflow, PreviousWorkflowDto dto, CancellationToken cancellationToken)
        {
            ValidatePermission(workflow);
            CheckWhetherRunnable(workflow.Status);
            var (currentStepBox, currentStep) = GetUnCompleteStep(workflow.StepBoxes, _sessionContext.RequiredOrgCode, _sessionContext.RequiredUserId);
            if (currentStepBox.StepType is EStepType.Start)
                throw UserFriendlyException.SameMessage("当前流程已退回到开始节点");
            if (currentStepBox.Steps.Count > 1)
                throw UserFriendlyException.SameMessage("会签流程不支持退回");

            //update trace
            await PreviousTraceAsync(workflow.Id, dto, currentStep, cancellationToken);

            //remove workflow.steps
            await _workflowStepRepository.RemoveRangeAsync(new List<WorkflowStep> { currentStepBox, currentStep },
                cancellationToken);

            //todo publish
        }

        /// <summary>
        /// 撤回（返回到之前任意节点）
        /// </summary>
        public async Task RecallAsync(Workflow workflow, RecallDto dto, CancellationToken cancellationToken)
        {
            ValidatePermission(workflow);
            CheckWhetherRunnable(workflow.Status);
            var targetStepBox = workflow.StepBoxes.FirstOrDefault(d => d.Code == dto.TargetStepCode);
            if (targetStepBox is null)
                throw UserFriendlyException.SameMessage("该流程尚未流转至该节点");

            await RecallAsync(workflow, dto, targetStepBox, cancellationToken);

            //todo publish
        }

        /// <summary>
        /// 跳转（直接将流程跳转至任意节点）
        /// </summary>
        public async Task JumpAsync(Workflow workflow, RecallDto dto, CancellationToken cancellationToken)
        {
            CheckWhetherRunnable(workflow.Status);
            //update uncompleted traces
            await JumpTraceAsync(workflow.Id, dto, cancellationToken);

            var (currentStepBox, currentStep) = GetUnCompleteStep(workflow.StepBoxes, _sessionContext.RequiredOrgCode, _sessionContext.RequiredUserId);
            if (currentStepBox.StepType is EStepType.Start)
                throw UserFriendlyException.SameMessage("当前流程已退回到开始节点");
            if (currentStepBox.StepType is EStepType.End)
                throw UserFriendlyException.SameMessage("当前流程已流转到结束节点");

            var targetStepBox = workflow.StepBoxes.FirstOrDefault(d => d.Code == dto.TargetStepCode);
            if (targetStepBox == null)
            {
                //向后跳转
                var nextStepBoxDefine = GetStepBoxDefine(workflow.Definition, dto.NextStepCode);
                var isStartCountersign = nextStepBoxDefine.IsStartCountersign(dto.NextHandlers.Count);
                var nextStepBox = await CreateStepAsync(isStartCountersign, workflow, nextStepBoxDefine, dto, EWorkflowStepStatus.Assigned, currentStepBox, currentStep, cancellationToken);

                await ResetWorkflowCurrentStepInfo(workflow, dto, nextStepBox, cancellationToken);

                #region 补充中间节点处理方案

                //var completeStepCodes = workflow.StepBoxes.Select(d => d.Code);
                //var uncompleteStepDefines = workflow.Definition.Steps.Where(d => !completeStepCodes.Contains(d.Code));
                //创建当前节点与目标节点中间节点
                //var jumpDto = new BasicWorkflowDto
                //{
                //    Opinion = "跳转补充"
                //};

                //foreach (var stepDefine in uncompleteStepDefines)
                //{
                //    var previousStepId = lastStepBox.Steps.Count > 1 ? lastStepBox.Id : lastStepBox.Steps.First().Id;
                //    if (dto.TargetStepCode == stepDefine.Code)
                //    {
                //        await CreateStepAsync(workflow, stepDefine, dto, lastStepBox.Id, previousStepId, cancellationToken);
                //        break;
                //    }

                //    //jump业务下，如果当前节点为会签节点，第一个补充节点的subStep.PreviousId无法确定从哪个子节点跳转过来，统一处理为当前节点的stepBox.Id
                //    lastStepBox = await CreateStepAsync(workflow, stepDefine, dto, lastStepBox.Id, previousStepId, cancellationToken);
                //} 

                #endregion
            }
            else
            {
                //返回之前节点
                await RecallAsync(workflow, dto, targetStepBox, cancellationToken);
            }

            //todo publish
        }

        /// <summary>
        /// 补充
        /// </summary>
        /// <returns></returns>
        public async Task SupplementAsync(Workflow workflow, EndWorkflowDto dto, CancellationToken cancellationToken)
        {
            CheckWhetherRunnable(workflow.Status);
            //todo 检查当前办理人是否为该流程中的办理人

            var supplement = _mapper.Map<WorkflowSupplement>(dto);
            await _workflowSupplementRepository.AddAsync(supplement, cancellationToken);
        }

        /// <summary>
        /// 终止流程
        /// </summary>
        public async Task TerminateAsync(string id, CancellationToken cancellationToken)
        {
            var workflow = await _workflowRepository.GetAsync(id, cancellationToken);
            if (workflow == null)
                throw UserFriendlyException.SameMessage("无效的流程编号");
            workflow.Terminate();
            await _workflowRepository.UpdateAsync(workflow, cancellationToken);

            //todo publish
            _mediator.Publish(new TerminalWorkflowNotify(workflow));
        }

        /// <summary>
        /// 根据stepCode查询流程配置中对应的节点
        /// </summary>
        public StepDefine GetStepBoxDefine(Definition definition, string stepCode)
        {
            if (definition == null) throw new ArgumentNullException(nameof(definition));
            if (string.IsNullOrEmpty(stepCode)) throw new ArgumentNullException(nameof(stepCode));
            var stepDefine = definition.FindStep(stepCode);
            if (stepDefine == null)
                throw new UserFriendlyException($"未找到流程中对应的节点，DefineCode: {definition.Code}, stepCode: {stepCode}",
                    "未查询到对应节点");
            return stepDefine;
        }

        /// <summary>
        /// 查询当前待办节点的下一级节点配置(办理参数)
        /// </summary>
        public IReadOnlyList<StepDefine> GetNextStepOptions(Workflow workflow, CancellationToken cancellationToken)
        {
            var (currentStepBox, _) = GetUnCompleteStep(workflow.StepBoxes, _sessionContext.RequiredOrgCode, _sessionContext.RequiredUserId);
            return workflow.Definition.FindSteps(currentStepBox.NextSteps);
        }


        #region private

        private async Task<WorkflowCountersign> CreateCountersignAsync(string workflowId, string startStepId, string startStepCode, int count, string? parentId = null, CancellationToken cancellationToken = default)
        {
            var countersign = new WorkflowCountersign
            {
                WorkflowId = workflowId,
                StartStepId = startStepId,
                StartStepCode = startStepCode,
                Members = count,
                ParentId = parentId,
            };
            await _workflowCountersignRepository.AddAsync(countersign, cancellationToken);
            return countersign;
        }


        /// <summary>
        /// 更新下级汇总节点可办理状态
        /// </summary>
        /// <param name="nextStepBox"></param>
        /// <param name="currentStep"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        private async Task SetNextCountersignEndAssignedAsync(WorkflowStep nextStepBox, WorkflowStep currentStep, CancellationToken cancellationToken)
        {
            var nextSteps = currentStep.StepCountersignStatus is EStepCountersignStatus.InCountersign
                ? nextStepBox.Steps.Where(d => d.CountersignId == currentStep.CountersignId).ToList()
                : nextStepBox.Steps.Where(d => d.PreviousId == currentStep.Id).ToList();

            if (!nextSteps.Any())
                throw new UserFriendlyException($"未查询到下一节点, currentStepId: {currentStep.Id}");

            foreach (var nextStep in nextSteps)
            {
                nextStep.SetAssigned();
            }

            await _workflowStepRepository.UpdateRangeAsync(nextSteps, cancellationToken);
        }

        /// <summary>
        /// 在stepCode对应的stepBox中找到开启会签流程的节点
        /// </summary>
        private static WorkflowStep FindCountersignStartStep(Workflow workflow, string startCountersignStepCode, string startCountersignId)
        {
            var countersignStartStepBox = workflow.StepBoxes.First(d => d.Code == startCountersignStepCode);
            var countersignStartStep =
                countersignStartStepBox.Steps.First(d => d.StartCountersignId == startCountersignId);
            return countersignStartStep;
        }

        private async Task JumpTraceAsync(string workflowId, RecallDto dto, CancellationToken cancellationToken)
        {
            //未办理的traces
            var uncompleteTraces =
                await _workflowTraceRepository.QueryAsync(d =>
                    d.WorkflowId == workflowId && string.IsNullOrEmpty(d.UserId));
            foreach (var trace in uncompleteTraces)
            {
                trace.Jump(
                    _sessionContext.RequiredUserId,
                    _sessionContext.UserName,
                    _sessionContext.RequiredOrgCode,
                    _sessionContext.OrgName);
            }

            await _workflowTraceRepository.UpdateRangeAsync(uncompleteTraces, cancellationToken);
        }

        private async Task RecallTraceAsync(string workflowId, RecallDto dto, CancellationToken cancellationToken)
        {
            //未办理的traces
            var uncompleteTraces =
                await _workflowTraceRepository.QueryAsync(d =>
                    d.WorkflowId == workflowId && string.IsNullOrEmpty(d.UserId));
            foreach (var trace in uncompleteTraces)
            {
                trace.Recall(
                    _sessionContext.RequiredUserId,
                    _sessionContext.UserName,
                    _sessionContext.RequiredOrgCode,
                    _sessionContext.OrgName);
            }

            await _workflowTraceRepository.UpdateRangeAsync(uncompleteTraces, cancellationToken);
        }

        private async Task PreviousTraceAsync(string workflowId, PreviousWorkflowDto dto, WorkflowStep step, CancellationToken cancellationToken)
        {
            var trace = await GetWorkflowTraceAsync(workflowId, step.Id, cancellationToken);
            _mapper.Map(dto, trace);
            trace.Previous(
                _sessionContext.RequiredUserId,
                _sessionContext.UserName,
                _sessionContext.RequiredOrgCode,
                _sessionContext.OrgName);
            await _workflowTraceRepository.UpdateAsync(trace, cancellationToken);
        }

        private async Task NextTraceAsync(Workflow workflow, BasicWorkflowDto dto, WorkflowStep step, CancellationToken cancellationToken)
        {
            var trace = await GetWorkflowTraceAsync(workflow.Id, step.Id, cancellationToken);
            _mapper.Map(dto, trace);
            _mapper.Map(step, trace);
            //trace.ExpiredTime = workflow.ExpiredTime;
            //trace.TimeLimit = workflow.TimeLimit;
            await _workflowTraceRepository.UpdateAsync(trace, cancellationToken);
        }

        private async Task AcceptTraceAsync(Workflow workflow, WorkflowStep currentStepBox, WorkflowStep currentStep, CancellationToken cancellationToken)
        {
            var trace = _mapper.Map<WorkflowTrace>(currentStep);
            trace.Status = EWorkflowTraceStatus.Normal;
            trace.ExpiredTime = workflow.ExpiredTime;
            trace.TimeLimit = workflow.TimeLimit;

            //处于会签中的节点，其对应的trace.parentId赋值上级trace.Id
            if (currentStep.StepCountersignStatus is EStepCountersignStatus.InCountersign)
            {
                var parentTrace = await GetWorkflowTraceAsync(workflow.Id, currentStep.PreviousId, cancellationToken);
                trace.ParentId = parentTrace.Id;
            }

            await _workflowTraceRepository.AddAsync(trace, cancellationToken);
        }

        private async Task<WorkflowTrace> GetWorkflowTraceAsync(string workflowId, string stepId, CancellationToken cancellationToken)
        {
            var parentTrace = await _workflowTraceRepository.GetAsync(d =>
                d.WorkflowId == workflowId && d.StepId == stepId, cancellationToken);
            if (parentTrace == null)
                throw new UserFriendlyException($"未找到对应trace, workflowId: {workflowId}, stepId: {stepId}");
            return parentTrace;
        }

        private async Task RecallAsync(Workflow workflow, RecallDto dto, WorkflowStep targetStepBox, CancellationToken cancellationToken)
        {
            //update uncompleted traces
            await RecallTraceAsync(workflow.Id, dto, cancellationToken);

            //remove completedSteps include target self
            var completeStepBoxes = workflow.StepBoxes.Where(d =>
                d.Code == dto.TargetStepCode || d.CreationTime > targetStepBox.CreationTime);
            var removeSteps = new List<WorkflowStep>();
            foreach (var stepBox in completeStepBoxes)
            {
                removeSteps.Add(stepBox);
                removeSteps.AddRange(stepBox.Steps);
            }

            await _workflowStepRepository.RemoveRangeAsync(removeSteps, cancellationToken);

            //recreate targetStep
            var nextStepBoxDefine = GetStepBoxDefine(workflow.Definition, dto.NextStepCode);
            var isStartCountersign = nextStepBoxDefine.IsStartCountersign(dto.NextHandlers.Count);
            await CreateStepAsync(isStartCountersign, workflow, nextStepBoxDefine, dto, EWorkflowStepStatus.Assigned, targetStepBox, targetStepBox.Steps.First(), cancellationToken);

            //flow manage
            if (workflow.IsInCountersign())
            {
                var currentCountersignStepBox =
                    workflow.StepBoxes.First(d => d.Code == workflow.TopCountersignStepCode);
                //目标节点在初始会签节点之前或正好
                if (targetStepBox.Code == workflow.TopCountersignStepCode || targetStepBox.CreationTime < currentCountersignStepBox.CreationTime)
                    await ResetWorkflowCurrentStepInfo(workflow, dto, targetStepBox, cancellationToken);

            }
        }

        /// <summary>
        /// 重置currentStep信息
        /// </summary>
        private async Task ResetWorkflowCurrentStepInfo(Workflow workflow, RecallDto dto, WorkflowStep stepBox, CancellationToken cancellationToken)
        {
            //更新当前节点名称、时间、会签节点code
            workflow.CloseCountersignStatus();
            var isCountersign = dto.NextHandlers.Count > 1;
            workflow.SetWorkflowCurrentStepInfo(isCountersign, stepBox);
            await _workflowRepository.UpdateAsync(workflow, cancellationToken);
        }

        private static void CheckWhetherRunnable(EWorkflowStatus status)
        {
            if (status is not EWorkflowStatus.Runnable)
                throw new UserFriendlyException("当前流程状态不可继续流转");
        }

        private void ValidatePermission(Workflow workflow)
        {
            if (!workflow.CanHandle(_sessionContext.RequiredUserId, _sessionContext.RequiredOrgCode))
                throw new UserFriendlyException("无办理权限");
        }

        /// <summary>
        /// 创建开始节点（保存开始流程的办理意见，对应definition的start节点）
        /// </summary>
        private async Task<(WorkflowStep stepBox, WorkflowStep step)> CreateStartStepAsync(Workflow workflow, BasicWorkflowDto dto, bool isStartCountersign, CancellationToken cancellationToken)
        {
            if (workflow.StepBoxes.Any())
                throw UserFriendlyException.SameMessage("无法反复开始流程");

            var startStepDefinition = workflow.Definition.Steps.FirstOrDefault(d => d.StepType == EStepType.Start);
            if (startStepDefinition == null)
                throw new UserFriendlyException($"模板未配置开始节点, defineCode: {workflow.Definition.Code}", "模板未配置开始节点");

            var stepBox = CreateStepBox(workflow.Id, startStepDefinition, dto, string.Empty);
            await _workflowStepRepository.AddAsync(stepBox, cancellationToken);

            //start节点的办理人分类默认为用户，即为当前发起流程的操作员
            var handler = new IdName { Id = _sessionContext.RequiredUserId, Name = _sessionContext.UserName };
            await CreateStartSubStepAsync(handler, dto, stepBox, cancellationToken);
            return (stepBox, stepBox.Steps.First());
        }

        /// <summary>
        /// 创建节点（不含开始、结束节点）
        /// </summary>
        private async Task<WorkflowStep> CreateStepAsync(bool isPrevStartCountersign, Workflow workflow, StepDefine stepBoxDefine, BasicWorkflowDto dto, EWorkflowStepStatus status,
            WorkflowStep? prevStepBox = null, WorkflowStep? prevStep = null, CancellationToken cancellationToken = default)
        {
            if (stepBoxDefine.StepType is EStepType.Start or EStepType.End)
                throw new UserFriendlyException("该方法不支持创建开始或结束节点");
            var stepBox = workflow.StepBoxes.FirstOrDefault(d => d.Code == stepBoxDefine.Code);
            if (stepBox == null)
            {
                stepBox = CreateStepBox(workflow.Id, stepBoxDefine, dto, prevStepBox?.Id ?? string.Empty);
                await _workflowStepRepository.AddAsync(stepBox, cancellationToken);
            }
            else if (stepBox.Status != EWorkflowStepStatus.Created)
            {
                stepBox.Status = EWorkflowStepStatus.Created;
                await _workflowStepRepository.UpdateAsync(stepBox, cancellationToken);
            }

            //下一节点为汇总节点时，同一会签只需要创建一次汇总节点
            if (stepBoxDefine.StepType is EStepType.CountersignEnd && prevStep.StepCountersignStatus == EStepCountersignStatus.InCountersign)
            {
                var step = stepBox.Steps.FirstOrDefault(d =>
                    d.IsInCountersign && d.CountersignId == prevStep.CountersignId);
                if (step != null)
                    return stepBox;
            }

            await CreateSubStepsAsync(isPrevStartCountersign, stepBoxDefine, dto, stepBox, status, prevStep, cancellationToken);

            return stepBox;
        }

        private async Task CreateStartSubStepAsync(
            IdName handler,
            BasicWorkflowDto dto,
            WorkflowStep stepBox,
            CancellationToken cancellationToken)
        {
            //开始节点既不发起会签，也不处于会签中
            var subStep = CreateSubStep(stepBox, new List<IdName> { handler }, dto.NextStepCode, dto.NextMainHandler,
                null, null, EWorkflowStepStatus.Completed, EStepCountersignStatus.None);
            subStep.Accept(_sessionContext.RequiredUserId, _sessionContext.UserName,
                _sessionContext.RequiredOrgCode, _sessionContext.OrgName);
            //step办理状态
            subStep.StepComplete(
                _sessionContext.RequiredUserId, _sessionContext.UserName,
                _sessionContext.RequiredOrgCode, _sessionContext.OrgName,
                dto.NextStepCode);
            _mapper.Map(dto, subStep);

            stepBox.Steps.Add(subStep);
            await _workflowStepRepository.AddAsync(subStep, cancellationToken);
        }

        private async Task CreateSubStepsAsync(
            bool isPrevStartCountersign,
            StepDefine stepBoxDefine,
            BasicWorkflowDto dto,
            WorkflowStep stepBox,
            EWorkflowStepStatus stepStatus,
            WorkflowStep prevStep,
            CancellationToken cancellationToken = default)
        {
            var countersignStatus = prevStep.GetNextStepCountersignStatus();
            List<WorkflowStep> subSteps;
            if (stepBoxDefine.HandlerType is EHandlerType.AssignUser or EHandlerType.AssignOrg)
            {
                subSteps = CreateSubSteps(isPrevStartCountersign, stepBox, stepBox.HandlerClassifies, dto.NextStepCode, dto.NextMainHandler,
                    prevStep?.Id, prevStep?.StartCountersignId, stepStatus, countersignStatus);
            }
            else
            {
                if (stepBoxDefine.HandlerType != EHandlerType.Role && !dto.NextHandlers.Any())
                    throw new UserFriendlyException("未指定节点处理者");
                subSteps = CreateSubSteps(isPrevStartCountersign, stepBox, dto.NextHandlers, dto.NextStepCode, dto.NextMainHandler,
                    prevStep?.Id, prevStep?.StartCountersignId, stepStatus, countersignStatus);
            }
            stepBox.Steps.AddRange(subSteps);
            await _workflowStepRepository.AddRangeAsync(subSteps, cancellationToken);
        }

        private (WorkflowStep stepBox, WorkflowStep step) GetStep(List<WorkflowStep> stepBoxes, string stepId)
        {
            foreach (var stepBox in stepBoxes)
            {
                foreach (var step in stepBox.Steps)
                {
                    if (step.Id == stepId)
                        return (stepBox, step);
                }
            }

            throw new UserFriendlyException("未找到对应节点");
        }

        /// <summary>
        /// 查询未完成节点
        /// </summary>
        /// <param name="stepBoxes"></param>
        /// <param name="orgCode"></param>
        /// <param name="userId"></param>
        /// <returns></returns>
        private (WorkflowStep, WorkflowStep) GetUnCompleteStep(List<WorkflowStep> stepBoxes, string orgCode, string userId)
        {
            var (stepBox, step) = GetStep(stepBoxes, orgCode, userId, d => d != EWorkflowStepStatus.Completed);
            if (step == null)
                throw new UserFriendlyException("未找到对应节点");
            return (stepBox, step);
        }

        private (WorkflowStep, WorkflowStep) GetUnCompleteStepOrDefault(List<WorkflowStep> stepBoxes, string orgCode, string userId) =>
            GetStep(stepBoxes, orgCode, userId, d => d != EWorkflowStepStatus.Completed);

        private (WorkflowStep, WorkflowStep) GetStep(List<WorkflowStep> stepBoxes, string orgCode, string userId, Func<EWorkflowStepStatus, bool> predicate)
        {
            if (!stepBoxes.Any()) throw new UserFriendlyException("该流程中暂无节点");
            foreach (var stepBox in stepBoxes)
            {
                foreach (var step in stepBox.Steps)
                {
                    if (predicate(step.Status) && (step.HandlerIds.Contains(orgCode) || step.HandlerIds.Contains(userId)))
                        return (stepBox, step);
                }
            }

            return new();
        }

        private WorkflowStep CreateStepBox(string workflowId, StepDefine stepDefine, BasicWorkflowDto dto, string prevStepBoxId)
        {
            var stepBox = _mapper.Map<WorkflowStep>(stepDefine);
            stepBox.WorkflowId = workflowId;
            stepBox.PreviousId = prevStepBoxId;
            stepBox.NextStepCode = string.Empty;
            stepBox.Opinion = string.Empty;
            return stepBox;
        }

        private List<WorkflowStep> CreateSubSteps(
            bool isPrevStartContersign,
            WorkflowStep stepBox,
            List<IdName> handlers,
            string nextStepCode,
            string? nextMainHandler,
            string? prevStepId,
            string? countersignId,
            EWorkflowStepStatus stepStatus,
            EStepCountersignStatus countersignStatus)
        {
            if (countersignStatus is EStepCountersignStatus.None && !string.IsNullOrEmpty(countersignId))
                throw UserFriendlyException.SameMessage("非法参数");
            if (countersignStatus is not EStepCountersignStatus.None && string.IsNullOrEmpty(countersignId))
                throw UserFriendlyException.SameMessage("非法参数");

            //依据是否发起会签创建step，发起会签表示一个handler创建一个step，未发起会签表示多人处理同一个节点，只创建一个step
            var steps = new List<WorkflowStep>();
            if (isPrevStartContersign)
            {
                foreach (var handler in handlers)
                {
                    var step = CreateSubStep(stepBox, new List<IdName> { handler }, nextMainHandler, nextStepCode,
                        prevStepId, countersignId, stepStatus, countersignStatus);

                    steps.Add(step);
                }
            }
            else
            {
                var step = CreateSubStep(stepBox, handlers, nextMainHandler, nextStepCode,
                    prevStepId, countersignId, stepStatus, countersignStatus);

                steps.Add(step);
            }

            return steps;
        }

        private WorkflowStep CreateSubStep(
            WorkflowStep stepBox,
            List<IdName> handlers,
            string nextStepCode,
            string? nextMainHandler,
            string? prevStepId,
            string? countersignId,
            EWorkflowStepStatus stepStatus,
            EStepCountersignStatus countersignStatus)
        {
            if (!handlers.Any())
                throw new UserFriendlyException("非法参数");
            var step = _mapper.Map<WorkflowStep>(stepBox);
            var handlerIds = handlers.Select(d => d.Id).ToList();
            var isMain = handlers.Count > 1 || handlerIds.First() == nextMainHandler;

            step.ParentId = stepBox.Id;
            step.HandlerIds = handlerIds;
            step.NextStepCode = nextStepCode;
            step.IsMain = isMain;
            step.PreviousId = prevStepId;
            step.CountersignId = countersignId;
            step.Status = stepStatus;
            step.StepCountersignStatus = countersignStatus;

            return step;
        }

        /// <summary>
        /// 依据配置生成过期时间
        /// </summary>
        /// <returns></returns>
        private DateTime GenerateExpiredTime(string defineCode)
        {
            //GetConfig(string defineCode).Time
            return DateTime.Now.AddDays(7); //todo 依据配置生成, Think about 工作日
        }

        private string GetTimeLimit(string defineCode)
        {
            //return GetConfig(string defineCode).Description;
            return "7个工作日";
        }

        //private ConfigInCludeDescriptionAndTime GetConfig(string defineCode)
        //{
        //    throw new NotImplementedException();
        //}

        #endregion

    }
}
